!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!                                                                                      !
!                   Various subroutines related to growth and remodeling               !
!                                                                                      !
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!!!!!!COMPUTE THE EVOLUTION OF GLYCOSAMINOGLYCAN APPERENT DENSITY AND STRESS-FREE STATE

      subroutine GAG_GNR(rho,Fr,Fe,Fe0,ksig,T,volfrac,mu,dt,GNR_switch)
      use operations
      implicit none
      real *8 rho,Fr(3,3),Fe(3,3),Fe0(3,3),ksig,T,volfrac,mu,dt,JJ,rho_dot,
     1        sigma_cur(3,3),sigma_hom(3,3),Be_cur(3,3),Be_hom(3,3),tmp1,
     2        F(3,3),C(3,3),Fr_dot(3,3),S_cur(3,3),S_hom(3,3)
      integer GNR_switch

      F=matmul(Fe,Fr)

      if (GNR_switch.eq.1) then !stress homeostasis
        JJ=det(F)
        call NH_cauchy_stress(rho,volfrac,mu,Fe,JJ,sigma_cur) !compute current GAG stress
        call NH_cauchy_stress(rho,volfrac,mu,Fe0,JJ,sigma_hom)!compute homeostatic GAG stress
        rho_dot=rho*ksig*(trace(sigma_cur)-trace(sigma_hom))/trace(sigma_hom) !time derivative of GAG density
      else !stretch homeostasis
        Be_cur=matmul(Fe,transpose(Fe))   !compute current GAG deformation
        Be_hom=matmul(Fe0,transpose(Fe0)) !compute homeostatic GAG deformation
        rho_dot=rho*ksig*(trace(Be_cur)-trace(Be_hom))/trace(Be_hom) !time derivative of GAG density
      endif

      !compute change in stress-free state
      tmp1=volfrac/(2.d0*mu*rho)*(rho_dot/rho+1.d0/T)
      call NH_2PK_stress(rho,volfrac,mu,Fr,Fe,S_cur)  !current GAG 2PK stress
      call NH_2PK_stress(rho,volfrac,mu,Fr,Fe0,S_hom) !homeostatic GAG 2PK stress

      C=matmul(transpose(F),F)
      Fr_dot=tmp1*matmul(matmul(Fr,(S_cur-S_hom)),C)

      !time integration using forward euler
      rho=rho+dt*rho_dot
      Fr=Fr+dt*Fr_dot

      return
      end

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!!!!!!COMPUTE THE EVOLUTION OF COLLAGEN APPERENT DENSITY AND STRESS-FREE STATE !!!!!!!!!!

      subroutine Collagen_GnR(rho,lr,le,le0,F,ksig,T,volfrac,k1,k2,k3,dt,GNR_switch)
      use operations
      implicit none
      real *8 rho, lr, le, le0, ksig, T, volfrac,k1,k2,k3,dt,F(3,3),
     1        rho_dot,sigma_cur,sigma_hom,S_cur,S_hom,tangent,tmp1,tmp5,
     2        lr_dot
      integer GNR_switch

      if (GNR_switch.eq.1) then !stress homeostasis
          call Collagen_scalar_cauchy_stress(rho,volfrac,F,le,k1,k2,k3,sigma_cur)  !current collagen stress
          call Collagen_scalar_cauchy_stress(rho,volfrac,F,le0,k1,k2,k3,sigma_hom) !homeostatic collagen stress
          rho_dot=rho*ksig*(sigma_cur-sigma_hom)/sigma_hom   !time derivative of collagen density
      else !stretch homeostasis
          rho_dot=rho*ksig*(le-le0)/le0 !time derivative of collagen density
      endif

      !compute change in stress-free state
      tmp1=rho_dot/rho+1.d0/T
      call Collagen_scalar_2PK_stress(rho,volfrac,k1,k2,k3,le,lr,S_cur)  !current collagen 2PK stress
      call Collagen_scalar_2PK_stress(rho,volfrac,k1,k2,k3,le0,lr,S_hom) !homeostatic collagen 2PK stress
      call Collagen_scalar_elastic_tangent(rho,volfrac,k1,k2,k3,le,lr,tangent) !derivative of collagen 2PK stress to collagen elastic stretch squared
      tmp5=lr/le**2
      lr_dot=tmp5*tmp1*(S_cur-S_hom)/tangent

      !time integration using forward euler
      rho=rho+dt*rho_dot
      lr=lr+dt*lr_dot

      return
      end

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
