!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!                                                                                      !
!                   Various subroutines related to hyperelasticity                     !
!                                                                                      !
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!!!!!!COMPUTE THE NEOHOOKEAN CAUCHY STRESS !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

      subroutine NH_cauchy_stress(rho,volfrac,mu,Fe,JJ,sigma)
      use operations
      implicit none
      real *8 rho,volfrac,mu,Fe(3,3),JJ,sigma(3,3),I2(3,3),Be(3,3)

      I2=matI2(3)
      Be=matmul(Fe,transpose(Fe))

      sigma=mu*rho/volfrac/JJ*(Be-I2)

      return
      end
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!!!!!!COMPUTE THE NEOHOOKEAN SECOND PIOLA-KIRCHOFF STRESS !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

      subroutine NH_2PK_stress(rho,volfrac,mu,Fr,Fe,S)
      use operations
      implicit none
      real *8 rho, volfrac, mu, Fr(3,3), Fe(3,3), S(3,3),
     1        F(3,3), C(3,3), Cr(3,3)

      F=matmul(Fe,Fr)
      C=matmul(transpose(F),F)
      Cr=matmul(transpose(Fr),Fr)

      S=mu*rho/volfrac*(inv(Cr)-inv(C))

      return
      end

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!!!!!!COMPUTE THE NEOHOOKEAN STIFFNESS TENSOR !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

      subroutine NH_tangent(rho,mu,JJ,CC)
      use operations
      implicit none
      real *8 rho, mu, JJ, CC(3,3,3,3),I2(3,3)

      I2=matI2(3)
      CC=mu*rho/JJ*(dyadpr22up(I2,I2)+dyadpr22low(I2,I2))

      return
      end

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!!!!!!DETERMINE THE ELASTIC STRETCH AND DEFORMED FIBER DIRECTION FOR A COLLAGEN FIBER !!!!!

      subroutine determine_direction_and_stretch(F,v_0,lr,le,v_f)
      use operations
      implicit none
      real *8 F(3,3), v_0(3),lr,le,v_f(3),lambda

      v_f=matmul(F,v_0)
      lambda=sqrt(inpr11(v_f,v_f))
      v_f=v_f/lambda
      le=lambda/lr

      return
      end

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!!!!!!COMPUTE THE SCALAR CAUCHY STRESS OF THE COLLAGEN COMPONENT !!!!!!!!!!!!!!!!!!!!!!!!!!

      subroutine Collagen_scalar_cauchy_stress(rho,volfrac,F,le,k1,k2,k3,sigma)
      use operations
      implicit none
      real *8 rho, volfrac,F(3,3),le,k1,k2,k3,sigma,JJ

      JJ=det(F)
      if (le.gt.1.d0) then
         sigma=rho/JJ/volfrac*k1*(exp(k2*(le**2-1.d0))-1.d0)*le**2
      else
         sigma=rho/JJ/volfrac*k1*k2/k3*(exp(k3*(le**2-1.d0))-1.d0)
      endif

      return
      end

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!!!!!!COMPUTE THE SCALAR SECOND PIOLA-KIRCHOFF STRESS OF THE COLLAGEN COMPONENT !!!!!!!!!!

      subroutine Collagen_scalar_2PK_stress(rho,volfrac,k1,k2,k3,le,lr,S)
      use operations
      implicit none
      real *8 rho, volfrac,k1,k2,k3,le,lr,S

      if (le.gt.1.d0) then
        S=rho/volfrac*k1*(exp(k2*(le**2-1.d0))-1.d0)/lr**2
      else
        S=rho/volfrac*k1*k2/k3*(exp(k3*(le**2-1.d0))-1.d0)/lr**2/le**2
      endif

      return
      end

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!!!!!!COMPUTE THE SCALAR ELASTIC TANGENT OF THE COLLAGEN COMPONENT !!!!!!!!!!!!!!!!!!!!!!

      subroutine Collagen_scalar_elastic_tangent(rho,volfrac,k1,k2,k3,le,lr,tangent)
      use operations
      implicit none
      real *8 rho, volfrac,k1,k2,k3,le,lr,tangent

      if (le.gt.1.d0) then
        tangent=2.d0*rho/volfrac*k1*k2*exp(k2*(le**2-1.d0))/lr**2
      else
        tangent=2.d0*rho/volfrac*k1*k2/k3*(k3*le**2*exp(k3*(le**2-1.d0))
     1      -exp(k3*(le**2-1.d0))+1.d0)/(lr**2*le**4)
      endif

      return
      end

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!!!!!!COMPUTE THE SCALAR EULARIAN TANGENT OF THE COLLAGEN COMPONENT !!!!!!!!!!!!!!!!!!!!!!

      subroutine Collagen_scalar_eularian_tangent(rho,JJ,volfrac,k1,k2,k3,le,tangent)
      use operations
      implicit none
      real *8 rho, volfrac,JJ,k1,k2,k3,le,tangent

      if (le.gt.1.d0) then
        tangent=2d0*rho/volfrac/JJ*k1*k2*exp(k2*(le**2-1.d0))*le**4
      else
        tangent=2.d0*rho/volfrac/JJ*k1*k2/k3*(exp(k3*(le**2-1.d0))*
     1      (k3*le**2-1.d0)+1.d0)
      endif

      return
      end

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!!!!!!COMPUTE THE PENALTY STRESS !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

      subroutine Penalty_stress(sigma,k,JJ,rho0,rho)
      use operations
      implicit none
      real *8 sigma(3,3), k, JJ, rho0, rho, Jm,I2(3,3)

      Jm=rho/rho0
      I2=matI2(3)

      sigma=k/(2.d0*Jm)*(JJ/Jm-1.d0)*I2

      return
      end
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!!!!!!COMPUTE THE PENALTY tangent !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

      subroutine Penalty_tangent(CC,k,JJ,rho0,rho)
      use operations
      implicit none
      real *8 CC(3,3,3,3), k, JJ, rho0, rho, Jm,I2(3,3)

      Jm=rho/rho0
      I2=matI2(3)

      CC=k/(2.d0*JJ)*(2.d0*JJ**2/Jm**2-JJ/Jm)*dyadpr22(I2,I2)-
     *   k/(2.d0*JJ)*(JJ**2/Jm**2-JJ/Jm)*(dyadpr22up(I2,I2)+dyadpr22low(I2,I2))

      return
      end
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!!!!!!CORRECTION OF THE ROTATION TO DETERMINE FIBER ANGLES IN CURRENT CONFIGURATION !!!!!!

      subroutine find_corrected_angle(v_c0,v_f0,F,angle)
      use operations
      implicit none
      real *8 v_c0(3),v_f0(3),F(3,3),angle,v_c(3),v_f(3)

      v_c=matmul(F,v_c0)
      v_c=v_c/sqrt(inpr11(v_c,v_c)) !deformed circumferential direction
      v_f=matmul(F,v_f0)
      v_f=v_f/sqrt(inpr11(v_f,v_f)) !deformed collagen direction
      angle=acos(inpr11(v_c,v_f))   !angle between the directions

      return
      end

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!