      include 'operations.f'
      include 'hyperelastic_subroutines.f'
      include 'GNR_subroutines.f'
      include 'sdvini_directions.f'
      include 'UVARM.f'

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!        states of the homogenized constrained mixture model                   !
!                                                                              !
!        statev(1)=elastin density                                             !
!        statev(2)=GAG density                                                 !
!        statev(3:nfib+2)=fiber densities                                      !
!        statev(nfib+3:nfib+11)=elastin remodeling stretches                   !
!        statev(nfib+12:nfib+20)=GAG remodeling stretches                      !
!        statev(nfib+21:2*nfib+20)=fiber remodeling stretch                    !
!        statev(2*nfib+21:3*nfib+20)=elastic fiber stretches                   !
!        statev(3*nfib+21:3*nfib+29)=elastic GAG stretches                     !
!        statev(3*nfib+30:3*nfib+38)=homeostatic GAG elastic deformation       !
!        statev(3*nfib+39:4*nfib+38)=homeostatic fiber stretches               !
!        statev(4*nfib+39:4*nfib+41)=circumferential direction                 !
!        statev(4*nfib+42:4*nfib+44)=radial direction                          !
!        statev(4*nfib+45:5*nfib+44)=deformation corrected angle               !
!                                                                              !
!        the documentation of this code is written in the file                 !
!             "development of a heart valve hcmm.pdf"                          !
!                                                                              !
!        in the first step prestretch and the homeostatic loads                !
!        should be introduced and defined, in step 2, you can impose           !
!        the pertubation of loading that induces GnR, the actual               !
!        growth and remodeling will then take place from step 3 onward         !
!                                                                              !
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

      subroutine umat(stress,statev,ddsdde,sse,spd,scd,
     *   rpl,ddsddt,drplde,drpldt,
     *   stran,dstran,time,dtime,temp,dtemp,predef,dpred,cmname,
     *   ndi,nshr,ntens,nstatv,props,nprops,coords,drot,pnewdt,
     *   celent,dfgrd0,dfgrd1,noel,npt,layer,kspt,kstep,kinc)
      
      use operations
      include 'aba_param.inc'

      character*80 cmname
      PARAMETER(nfib=30)
      dimension stress(ntens),statev(nstatv),
     *   ddsdde(ntens,ntens),ddsddt(ntens),drplde(ntens),
     *   stran(ntens),dstran(ntens),time(2),predef(1),dpred(1),
     *   props(nprops),coords(3),drot(3,3),dfgrd0(3,3),dfgrd1(3,3)

      real*8 sig(3,3), F(3,3), Fr_e(3,3),Fr_g(3,3),lambda_r(nfib),volfrac_f(nfib),
     1       rho_R(nfib),v_circ(3), v_rad(3),v_fib_0(nfib,3),I2(3,3),sigma_e(3,3),
     2       sigma_g(3,3),CC_e(3,3,3,3),CC_g(3,3,3,3),v_fib_f(3),sigma_fib(nfib,3,3),
     3       CC_fib_e(nfib,3,3,3,3), C(3,3,3,3),sigma_penalty(3,3),CC_penalty(3,3,3,3),
     4       gamma(nfib),Fe_e(3,3),Fe_g(3,3),Be_e(3,3),Be_g(3,3),le_prev(nfib),
     5       lambda_e(nfib),Fe_prev(3,3),F_prev(3,3),C_prev(3,3),le_0(nfib),
     6       Fr_dot(3,3),Cr_g(3,3),Fe_g0(3,3)

      real*8 mu_e, k1, k2,k3, mu_g, kappa, alpha, beta, rho, volfrac_e,volfrac_g,
     1       volfrac_fib,summed_volfracs,rho_R_e,rho_R_g,pi,JJ,
     2       lambda,ksig, Tfib, lr0,Tgag,ksigG,rho_tot,sigf,CCf
      integer ndim, i, j, aa, bb, nfib, GNR_switch

!     initialisation of parameters !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

      GNR_switch=1 !type of homeostasis: 0 = stretch GnR 1 = stress GnR
      pi=2.d0*acos(0.0)
      ndim=3


!     MATERIAL PARAMETERS
      rho=1050.d0        !Total tissue reference density [kg/m3]
      mu_e=0.1190E-3     !Elastin shear modulus [MPa/kg/m3]
      k1=0.0017E-3       !Collagen stiffness parameter 1 [MPa/kg/m3]
      k2=5.245           !Collagen stiffness parameter 2 [-]
      k3=25.d0           !Collagen compressive stiffness parameter [-]
      mu_g=mu_e/1000.d0  !GAG shear modulus [MPa/kg/m3]
      kappa=0.24157d0    !penalty stiffness parameter [MPa]
      alpha=0.d0         !initial dominant fiber orientation with respect to circumferential direction [radians]
      beta=0.8287        !initial fiber dispersity [-]
      lr0=1.d0/1.25      !collagen remodeling stretch in the reference orientation [-]

!     growth and remodeling parameters
      Tfib=70.d0      !half-life of collagen [days]
      Tgag=45.d0      !half-life of GAGs [days]
      ksig=0.3/Tfib   !Gain parameter of collagen [days^-1]
      ksigG=0.3/Tgag  !Gain parameter of GAGs [days^-1]


!     determine the undeformed fiber directions
      v_circ=statev(4*nfib+39:4*nfib+41) !circumferential direction
      v_rad=statev(4*nfib+42:4*nfib+44)  !radial direction
      do i=1,nfib
      gamma(i)=dble(i)/dble(nfib)*pi-pi/2d0 !fiber angle with the circumferential direction
      v_fib_0(i,:)=cos(gamma(i))*v_circ+sin(gamma(i))*v_rad !undeformed fiber direction
      enddo


!     load constituent densities and volume fractions 

        ! load the initial volumefractions
        volfrac_e=0.20
        volfrac_g=0.35
        volfrac_fib=1.d0-volfrac_e-volfrac_g
        ! Determine the volumefraction of the individual fibers
        do i=1,nfib
          volfrac_f(i)=exp((cos(2.d0*(gamma(i)-alpha))+1.d0)/beta)+exp((cos(2.d0*(gamma(i)+alpha))+1.d0)/beta)
        enddo
        summed_volfracs=sum(volfrac_f)
        ! determine constituent apparent densities
        rho_R_e=volfrac_e*rho
        rho_R_g=volfrac_g*rho
        do i=1,nfib
          volfrac_f(i)=volfrac_f(i)/summed_volfracs*volfrac_fib
          rho_R(i)=volfrac_f(i)*rho
        enddo
      
        ! after the first step, overwrite these volumefractions and densities from the statevariables from the previous increment
      if (kstep.gt.1) then 
        rho_R_e=statev(1)
        rho_R_g=statev(2)
        rho_R=statev(3:2+nfib)
        rho_tot=rho_R_e+rho_R_g+sum(rho_R)
        volfrac_e=rho_R_e/rho_tot
        volfrac_g=rho_R_g/rho_tot
        do i=1,nfib
          volfrac_f(i)=rho_R(i)/rho_tot
        enddo
      endif

      I2=matI2(3) !identity tensor

!    load constituent stress-free states
      if (kstep.eq.1) then
         !in the first step (duration 1) gently introduce the prestretches by linearly increasing over time
         do i=1,nfib
            lambda_r(i)=1d0+(lr0-1d0)*(time(1)+dtime) !collagen fiber stress-free state
         enddo
           Fr_e(1,:)=statev(3+nfib:3+nfib+2)
           Fr_e(2,:)=statev(3+nfib+3:3+nfib+5)
           Fr_e(3,:)=statev(3+nfib+6:3+nfib+8)
           Fr_e=I2+(Fr_e-I2)*(time(1)+dtime) !elastin stress-free state
           Fr_g(1,:)=statev(12+nfib:12+nfib+2)
           Fr_g(2,:)=statev(12+nfib+3:12+nfib+5)
           Fr_g(3,:)=statev(12+nfib+6:12+nfib+8)
           Fr_g=I2+(Fr_g-I2)*(time(1)+dtime) !GAG stress-free state
      else !in the subsequent steps, load stress-free state from state variables
         do i=1,nfib
            lambda_r(i)=statev(20+i+nfib) !collagen fiber stress-free state
         enddo
         do i=1,3
           do j=1,3
              Fr_e(i,j)=statev(3*(i-1)+(j-1)+nfib+3)  !elastin stress-free state
              Fr_g(i,j)=statev(12+nfib+3*(i-1)+(j-1)) !GAG stress-free state
           enddo
         enddo
      endif

!    PREREQUISITES FOR G&R
!      current and homeostatic elastic stretches
       do i=1,nfib
          le_prev(i)=statev(2*nfib+20+i)
          le_0(i)=statev(3*nfib+38+i)
       enddo

!      current and homeostatic elastic GAG stretches
       do i=1,3
         do j=1,3
           Fe_prev(i,j)=statev(3*nfib+21+3*(i-1)+(j-1))
           Fe_g0(i,j)=statev(3*nfib+30+3*(i-1)+(j-1))
         enddo
       enddo

      F_prev=dfgrd0

! Growth and Remodeling !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

      if (KSTEP.gt.2.and.KSTEP.lt.4) then

! GAG GnR 
      call GAG_GNR(rho_R_g,Fr_g,Fe_prev,Fe_g0,ksigG,Tgag,volfrac_g,mu_g,dtime,GNR_switch)

! Collagen GnR
      do i=1,nfib
         call collagen_GnR(rho_R(i),lambda_r(i),le_prev(i),le_0(i),F_prev,ksig,Tfib,volfrac_f(i),k1,k2,k3,dtime,GNR_switch)
      enddo

      endif

! update volume fractions
      rho_tot=rho_R_e+rho_R_g+sum(rho_R)
      volfrac_e=rho_R_e/rho_tot
      volfrac_g=rho_R_g/rho_tot
      do i=1,nfib
        volfrac_f(i)=rho_R(i)/rho_tot
      enddo

! Hyperelasticity !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!     kinematics
      F=dfgrd1
      JJ=det(F)
      Fe_e=matmul(F,inv(Fr_e))
      Fe_g=matmul(F,inv(Fr_g))

!     elastin hyperelasticity
      call NH_cauchy_stress(rho_R_e,volfrac_e,mu_e,Fe_e,JJ,sigma_e)
      call NH_tangent(rho_R_e,mu_e,JJ,CC_e)

!     gag hyperelasticity
      call NH_cauchy_stress(rho_R_g,volfrac_g,mu_g,Fe_g,JJ,sigma_g)
      call NH_tangent(rho_R_g,mu_g,JJ,CC_g)

!     collagen hyperelasticity
      do i=1,nfib
        call determine_direction_and_stretch(F,v_fib_0(i,:),lambda_r(i),lambda_e(i),v_fib_f)
        call Collagen_scalar_cauchy_stress(rho_R(i),volfrac_f(i),F,lambda_e(i),k1,k2,k3,sigf)
        sigma_fib(i,:,:)=sigf*dyadpr11(v_fib_f,v_fib_f)
        call collagen_scalar_eularian_tangent(rho_R(i),JJ,volfrac_f(i),k1,k2,k3,lambda_e(i),CCf)
        CC_fib_e(i,:,:,:,:)=CCf*dyadpr22(dyadpr11(v_fib_f,v_fib_f),dyadpr11(v_fib_f,v_fib_f))
      enddo

!     penalty energy
      call Penalty_stress(sigma_penalty,kappa,JJ,rho,rho_tot)
      call Penalty_tangent(CC_penalty,kappa,JJ,rho,rho_tot)

!     assemble stress
      sig=volfrac_e*sigma_e+volfrac_g*sigma_g+sigma_penalty
      do i=1,nfib
         sig=sig+volfrac_f(i)*sigma_fib(i,:,:)
      enddo

!     assemble tangents
      C=CC_e+CC_g+CC_penalty
      do i=1,nfib
         C=C+volfrac_f(i)*CC_fib_e(i,:,:,:,:)
      enddo
      C=C+0.5*(dyadpr22up(I2,sig)+dyadpr22low(I2,sig)+dyadpr22up(sig,I2)+dyadpr22low(sig,I2))

! Update statevariables !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
      statev(1)=rho_R_e         !elastin apparent density
      statev(2)=rho_R_g         !GAG apparent density
      do i=1,nfib
        statev(2+i)=rho_R(i)    !collagen fiber density
      enddo

      if (kstep.gt.1) then
        do i=1,3
           do j=1,3
              statev(nfib+3+3*(i-1)+(j-1))=Fr_e(i,j)  !elastin stress-free state
              statev(nfib+12+3*(i-1)+(j-1))=Fr_g(i,j) !GAG stress-free state
           enddo
        enddo
      endif

      do i=1,nfib
            statev(nfib+20+i)=lambda_r(i) !collagen fiber stress-free state
            statev(2*nfib+20+i)=lambda_e(i) !current collagen fiber elastic stretch
      enddo
      do i=1,3
         do j=1,3
            statev(3*nfib+21+3*(i-1)+(j-1))=Fe_g(i,j) !current GAG elastic stretch
         enddo
      enddo
      if (KSTEP.eq.1) then !at the end of the first step (pulmonary valve under pulmonary conditions) we assume that the valve is in homeostasis
        do i=1,3
          do j=1,3
            statev(3*nfib+30+3*(i-1)+(j-1))=Fe_g(i,j) !GAG homeostatic target stretch
         enddo
        enddo
        do i=1,nfib
          if (lambda_e(i).le.1.1d0) then
            statev(3*nfib+38+i)=1.1d0 !lower boundary collagen homeostatic stretch
          else
            statev(3*nfib+38+i)=lambda_e(i) !collagen homeostatic stretch
          endif
        enddo
      endif

      do i=1,nfib
         call find_corrected_angle(v_circ,v_fib_0(i,:),dfgrd1,statev(4*nfib+44+i)) !deformed fiber angle wrt. circumferential direction (postprocessing purposes only) 
      enddo

      statev(5*nfib+45)=sqrt(inpr11(matmul(F,v_circ),matmul(F,v_circ))) !circumferential stretch (postprocessing purposes only) 
      statev(5*nfib+46)=sqrt(inpr11(matmul(F,v_rad),matmul(F,v_rad)))   !radial stretch (postprocessing purposes only) 

!     *** Update stress to voight notation
      stress(1) = sig(1,1)
      stress(2) = sig(2,2)
      stress(3) = sig(3,3)
      stress(4) = sig(1,2)
      stress(5) = sig(1,3)
      stress(6) = sig(2,3)

!     *** Update stiffness matrix to voight notation
      do i = 1,(2*ndim)
         if (i .le. ndim) then
            ddsdde(i,1) = C(i,i,1,1)
            ddsdde(i,2) = C(i,i,2,2)
            ddsdde(i,3) = C(i,i,3,3)
            ddsdde(i,4) = (C(i,i,1,2) + C(i,i,2,1))/2.0
            ddsdde(i,5) = (C(i,i,1,3) + C(i,i,3,1))/2.0
            ddsdde(i,6) = (C(i,i,2,3) + C(i,i,3,2))/2.0
         else
            aa = (i-2)/2
            bb = (i-1)/2 + 1
            ddsdde(i,1) = C(aa,bb,1,1)
            ddsdde(i,2) = C(aa,bb,2,2)
            ddsdde(i,3) = C(aa,bb,3,3)
            ddsdde(i,4) = (C(aa,bb,1,2) + C(aa,bb,2,1))/2.0
            ddsdde(i,5) = (C(aa,bb,1,3) + C(aa,bb,3,1))/2.0
            ddsdde(i,6) = (C(aa,bb,2,3) + C(aa,bb,3,2))/2.0
         endif
      enddo

      return
      end
