       MODULE operations  
       IMPLICIT real*8(a-h,o-z), integer(i,n)
       DIMENSION zeros2(3,3),zeros4(3,3,3,3),zeros6(6,6)
       DATA zeros2/9*0d0/ 
       DATA zeros4/81*0d0/ 	
       DATA zeros6/36*0d0/ 	   
	   
       CONTAINS      
c=======================================================================	
c	   second order unit tensor
c=======================================================================
        PURE FUNCTION matI2(ndim)

        implicit none
        INTENT(IN):: ndim
        integer ndim, i
        real*8 matI2(ndim,ndim)

        matI2=0d0
        do i=1,ndim
           matI2(i,i)=1d0
        enddo
        END FUNCTION matI2

c=======================================================================	
c	    fourth order unit tensor
c=======================================================================
        PURE FUNCTION matI4(ndim)

        implicit none
        INTENT(IN):: ndim
        integer ndim, i,j,k,l
        real*8 matI4(ndim,ndim,ndim,ndim),dil,djk,dik,djl

        do i = 1,ndim
           do j = 1,ndim
              do k = 1,ndim
                 do l = 1,ndim
                    dil = 0.0
                    djk = 0.0
                    dik = 0.0
                    djl = 0.0
                    if (i .eq. l) dil = 1.0
                    if (j .eq. k) djk = 1.0
                    if (i .eq. k) dik = 1.0
                    if (j .eq. l) djl = 1.0
                    matI4(i,j,k,l) = dil*djk
                 enddo
              enddo
           enddo
        enddo
        END FUNCTION matI4

c=======================================================================
c       fourth order symetric unit tensor
c=======================================================================

      PURE FUNCTION matI4s(ndim)
      implicit none
      intent(IN):: ndim
      integer ndim, i, j, k, l
      real*8 matI4S(ndim,ndim,ndim,ndim), dil, djk, dik, djl

      do i = 1,ndim
         do j = 1,ndim
            do k = 1,ndim
               do l = 1,ndim
                  dil = 0.0
                  djk = 0.0
                  dik = 0.0
                  djl = 0.0
                  if (i .eq. l) dil = 1.0
                  if (j .eq. k) djk = 1.0
                  if (i .eq. k) dik = 1.0
                  if (j .eq. l) djl = 1.0
                  matI4S(i,j,k,l) = 0.5d0*(dil*djk + dik*djl)
               enddo
            enddo
         enddo
      enddo
      end function matI4S

C=======================================================================
c                  determinant of a matrix
c=======================================================================
        PURE FUNCTION det(mat)
        implicit none
        intent(in) mat
        real*8 det, mat(3,3)

        det    = mat(1,1)*mat(2,2)*mat(3,3)
     *          + mat(1,2)*mat(2,3)*mat(3,1)
     *          + mat(1,3)*mat(2,1)*mat(3,2) 
     *          - mat(1,3)*mat(2,2)*mat(3,1)
     *          - mat(1,2)*mat(2,1)*mat(3,3) 
     *          - mat(1,1)*mat(2,3)*mat(3,2)

      end function

c=======================================================================
c                  inverse of a matrix
c=======================================================================
          PURE FUNCTION inv(mat)
          implicit none
          intent(IN):: mat
          real*8 inv(3,3), mat(3,3),deter
          deter=det(mat)
          inv(1,1) =  (mat(2,2)*mat(3,3) - mat(2,3)*mat(3,2))/deter
          inv(1,2) = -(mat(1,2)*mat(3,3) - mat(1,3)*mat(3,2))/deter
          inv(1,3) =  (mat(1,2)*mat(2,3) - mat(1,3)*mat(2,2))/deter
          inv(2,1) = -(mat(2,1)*mat(3,3) - mat(2,3)*mat(3,1))/deter
          inv(2,2) =  (mat(1,1)*mat(3,3) - mat(1,3)*mat(3,1))/deter
          inv(2,3) = -(mat(1,1)*mat(2,3) - mat(1,3)*mat(2,1))/deter
          inv(3,1) =  (mat(2,1)*mat(3,2) - mat(2,2)*mat(3,1))/deter
          inv(3,2) = -(mat(1,1)*mat(3,2) - mat(1,2)*mat(3,1))/deter
          inv(3,3) =  (mat(1,1)*mat(2,2) - mat(1,2)*mat(2,1))/deter
          END FUNCTION
c=======================================================================
c         inner-product of two first-order tensors
c=======================================================================
        PURE FUNCTION inpr11(A,B)	
          IMPLICIT real*8(a-h,o-z), integer(i,n)
          INTENT(IN):: A,B
          DIMENSION A(3),B(3)
          real*8 inpr11
          inpr11=0d0	
          do i=1,3
             inpr11=inpr11+A(i)*B(i)
          enddo	  	  
        END FUNCTION inpr11
c=======================================================================
c         inner-product of two second-order tensors
c=======================================================================
        PURE FUNCTION inpr22(A,B)	
          IMPLICIT real*8(a-h,o-z), integer(i,n)
          INTENT(IN):: A,B
          DIMENSION A(3,3),B(3,3)
          real*8 inpr22(3,3)
          do i=1,3
            do j=1,3
              inpr22(i,j)=0d0
            enddo
          enddo

          do i=1,3
            do j=1,3
              do k=1,3
                 inpr22(i,k)=inpr22(i,k)+A(i,j)*B(j,k)
               enddo
            enddo
          enddo	  	  
        END FUNCTION inpr22
c=======================================================================
c         Double inner-product of two fourth-order tensors
c=======================================================================
        PURE FUNCTION dinpr44(A,B)	
          IMPLICIT real*8(a-h,o-z), integer(i,n)
          INTENT(IN):: A,B
          DIMENSION A(3,3,3,3),B(3,3,3,3),dinpr44(3,3,3,3)	
          do i=1,3
           do j=1,3   
            do k=1,3 
             do l=1,3         
              dinpr44(i,j,k,l)=0d0	
              do m=1,3	
               do n=1,3			  
		        dinpr44(i,j,k,l)=dinpr44(i,j,k,l)+A(i,j,m,n)*
     *				             B(m,n,k,l)         
               enddo	
              enddo	
             enddo		   
            enddo		 
           enddo
          enddo	  	  
        END FUNCTION dinpr44

c=======================================================================
c         Double inner-product of a second and fourth-order tensor
c=======================================================================
        PURE FUNCTION dinpr24(A,B)
          IMPLICIT real*8(a-h,o-z), integer(i,n)	
          INTENT(IN):: A,B
          DIMENSION A(3,3),B(3,3,3,3),dinpr24(3,3)	     
          do i=1,3
           do j=1,3
            dinpr24(i,j)=0d0		   
            do k=1,3 
             do l=1,3 		
              dinpr24(i,j)=dinpr24(i,j)+A(k,l)*B(k,l,i,j)
             enddo		   
            enddo		 
           enddo
          enddo	  
        END FUNCTION dinpr24

c=======================================================================
c         Double inner-product of a fourth and second order tensor
c=======================================================================
        PURE FUNCTION dinpr42(A,B)	 	
          IMPLICIT real*8(a-h,o-z), integer(i,n)	
          INTENT(IN):: A,B
          DIMENSION A(3,3,3,3),B(3,3),dinpr42(3,3)	    
          do i=1,3
           do j=1,3
            dinpr42(i,j)=0d0		   
            do k=1,3 
             do l=1,3 		
		      dinpr42(i,j)=dinpr42(i,j)+A(i,j,k,l)*B(k,l)
             enddo		   
            enddo		 
           enddo
          enddo	  
        END FUNCTION dinpr42

c=======================================================================
c         Double inner-product of two second order tensors
c=======================================================================		
        PURE FUNCTION dinpr22(A,B)	

          IMPLICIT real*8(a-h,o-z), integer(i,n)
          INTENT(IN):: A,B  
          DIMENSION A(3,3),B(3,3)	  
          dinpr22=0d0    
          do i=1,3
           do j=1,3		   
		    dinpr22=dinpr22+A(i,j)*B(i,j)		 
           enddo
          enddo
        END FUNCTION dinpr22

c=======================================================================
c          Dyadic product of two first order tensors (vectors)
c=======================================================================	
        PURE FUNCTION dyadpr11(vec1,vec2)
        implicit none
        intent(IN):: vec1,vec2
        integer i,j
        real*8 vec1(3),vec2(3),dyadpr11(3,3)
        do i = 1,3
          do j = 1,3
               dyadpr11(i,j) = vec1(i)*vec2(j)
          enddo
        enddo
        END FUNCTION dyadpr11

c=======================================================================
c          Dyadic product of two second order tensors
c=======================================================================	
        PURE FUNCTION dyadpr22(mat1,mat2)
        implicit none
        intent(IN):: mat1,mat2
        integer i,j,k,l
        real*8 mat1(3,3),mat2(3,3),dyadpr22(3,3,3,3)
        do i = 1,3
          do j = 1,3
             do k = 1,3
                do l = 1,3
                   dyadpr22(i,j,k,l) = mat1(i,j)*mat2(k,l)
                enddo
             enddo
          enddo
        enddo
        END FUNCTION dyadpr22

c=======================================================================
c     Upper dyadic product of two second order tensors
c=======================================================================
        PURE FUNCTION dyadpr22up(mat1,mat2)
        implicit none
        intent(IN):: mat1,mat2
        integer i,j,k,l
        real*8 mat1(3,3),mat2(3,3),dyadpr22up(3,3,3,3)
        do i = 1,3
          do j = 1,3
             do k = 1,3
                do l = 1,3
                   dyadpr22up(i,j,k,l) = mat1(i,k)*mat2(j,l)
                enddo
             enddo
          enddo
        enddo
        END FUNCTION dyadpr22up
		
c=======================================================================
c     Lower dyadic product of two second order tensors
c=======================================================================
        PURE FUNCTION dyadpr22low(mat1,mat2)
        implicit none
        intent(IN):: mat1,mat2
        integer i,j,k,l
        real*8 mat1(3,3),mat2(3,3),dyadpr22low(3,3,3,3)
        do i = 1,3
          do j = 1,3
             do k = 1,3
                do l = 1,3
                   dyadpr22low(i,j,k,l) = mat1(i,l)*mat2(j,k)
                enddo
             enddo
          enddo
        enddo
        END FUNCTION dyadpr22low

c=======================================================================
c      multiway upper-lower-dyadic product combination
c=======================================================================
        PURE FUNCTION dyadpr22combi(mat1,mat2)
        IMPLICIT none
        intent(IN):: mat1,mat2
        integer i,j,k,l
        real*8 mat1(3,3),mat2(3,3),dyadpr22combi(3,3,3,3)
        do i = 1,3
          do j = 1,3
             do k = 1,3
                do l = 1,3
                   dyadpr22combi(i,j,k,l) = mat1(i,k)*mat2(j,l)+
     *  mat1(i,l)*mat2(j,k)+mat1(j,l)*mat2(i,k)+mat1(j,k)*mat2(i,l)
                enddo
             enddo
          enddo
        enddo

        end function dyadpr22combi
c=======================================================================
c       cross product of two vectors
c=======================================================================
        PURE FUNCTION crossproduct(vec1,vec2)
        IMPLICIT NONE
        intent(IN):: vec1,vec2
        real*8 vec1(3),vec2(3),crossproduct(3)
        crossproduct(1)=vec1(2)*vec2(3)-vec1(3)*vec2(2)
        crossproduct(2)=vec1(3)*vec2(1)-vec1(1)*vec2(3)
        crossproduct(3)=vec1(1)*vec2(2)-vec1(2)*vec2(1)
        end function crossproduct

c=======================================================================
c       trace of a second order tensor
c=======================================================================
        PURE FUNCTION trace(tens)
        IMPLICIT NONE
        intent(IN):: tens
        real*8 tens(3,3),trace
        integer i
             trace=0d0
             do i=1,3
                trace=trace+tens(i,i)
             enddo
        end function trace
c=======================================================================
c      convert voight notation to tensor notation
C=======================================================================
        PURE FUNCTION voight2tens(voight)
        IMPLICIT NONE
        intent(IN):: voight
        real*8 voight2tens(3,3),voight(6)
        voight2tens(1,1)=voight(1)
        voight2tens(2,2)=voight(2)
        voight2tens(3,3)=voight(3)
        voight2tens(1,2)=0.5*voight(4)
        voight2tens(2,3)=0.5*voight(5)
        voight2tens(1,3)=0.5*voight(6)
        voight2tens(2,1)=0.5*voight(4)
        voight2tens(3,2)=0.5*voight(5)
        voight2tens(3,1)=0.5*voight(6)
        end function voight2tens
      END MODULE operations	