!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!        states of the homogenized constrained mixture model                   !
!                                                                              !
!        statev(1)=elastin density                                             !
!        statev(2)=GAG density                                                 !
!        statev(3:nfib+2)=fiber densities                                      !
!        statev(nfib+3:nfib+11)=elastin remodeling stretches                   !
!        statev(nfib+12:nfib+20)=GAG remodeling stretches                      !
!        statev(nfib+21:2*nfib+20)=fiber remodeling stretch                    !
!        statev(2*nfib+21:3*nfib+20)=elastic fiber stretches                   !
!        statev(3*nfib+21:3*nfib+29)=elastic GAG stretches                     !
!        statev(3*nfib+30:3*nfib+38)=homeostatic GAG elastic deformation       !
!        statev(3*nfib+39:4*nfib+38)=homeostatic fiber stretches               !
!        statev(4*nfib+39:4*nfib+41)=circumferential direction                 !
!        statev(4*nfib+42:4*nfib+44)=radial direction                          !
!        statev(4*nfib+45:5*nfib+44)=deformation corrected angle               !
!        statev(5*nfib+45)=circumferential stretch                             !
!        statev(5*nfib+46)=axial stretch                                       !
!                                                                     !
!        the documentation of this code is written in the file        !
!             "development of a heart valve hcmm.pdf"                 !
!                                                                     !
!        in the first step prestretch and the homeostatic loads       !
!        should be introduced and defined, in step 2, you can impose  !
!        the pertubation of loading that induces GnR, the actual      !
!        growth and remodeling will then take place from step 3 onward!
!                                                                     !
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
      SUBROUTINE SDVINI(STATEV,COORDS,NSTATV,NCRDS,NOEL,NPT,
     1 LAYER,KSPT)

      use operations
      INCLUDE 'ABA_PARAM.INC'

      PARAMETER(NOELMAX=76540, nfib=30 )

      DIMENSION STATEV(NSTATV),COORDS(NCRDS), vcirc(3),vrad(3),vcirctot(3*NOELMAX),vradtot(3*NOELMAX),vt(3),
     1 Fe0(3,3), Fr_g(3,3), Fr_e(3,3)

      real*8 lambda_circ,lambda_rad
      integer i, rdstat, rdcnt, mydirlen
      character*256 mydirectory

      do i=1,NSTATV
         statev(i)=0d0
      enddo

      call getoutdir(mydirectory, mydirLen) !get the current directory

      lambda_circ=1.2 !circumferential prestretch GAGs and elastin
      lambda_rad=1.2  !radial prestretch GAGs and elastin

!     read circumferential direction
      open(unit=21, file=trim(mydirectory)//'/v1_el')
      rdstat = 0
      rdcnt = 1
      do while (rdstat.eq.0)
         read(21,*,iostat=rdstat) vcirctot(rdcnt)
         rdcnt = rdcnt +1
      enddo
      close(21)
      vcirc(1) = vcirctot((noel-1)*3 + 1)
      vcirc(2) = vcirctot((noel-1)*3 + 2)
      vcirc(3) = vcirctot((noel-1)*3 + 3)


!     read radial direction
      open(unit=22, file=trim(mydirectory)//'/v2_el')
      rdstat = 0
      rdcnt = 1
      do while (rdstat.eq.0)
         read(22,*,iostat=rdstat) vradtot(rdcnt)
         rdcnt = rdcnt +1
      enddo
      close(22)
      vrad(1) = vradtot((noel-1)*3 + 1)
      vrad(2) = vradtot((noel-1)*3 + 2)
      vrad(3) = vradtot((noel-1)*3 + 3)

!     return directions as statevariables
      statev(4*nfib+39:4*nfib+41)=vcirc
      statev(4*nfib+42:4*nfib+44)=vrad

!     GAG AND ELASTIN PRESTRETCHES !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

      vt=crossproduct(vcirc,vrad) !compute thickness direction
!     gags

      !initial GAG prestretch tensor
      Fe0=lambda_circ*dyadpr11(vcirc,vcirc)+ 
     *    lambda_rad*dyadpr11(vrad,vrad)+
     *    1d0/(lambda_rad*lambda_circ)*dyadpr11(vt,vt)

      Fr_g=inv(Fe0) !corresponding initial stress-free state
      statev(nfib+12:nfib+14)=Fr_g(1,:)
      statev(nfib+15:nfib+17)=Fr_g(2,:)
      statev(nfib+18:nfib+20)=Fr_g(3,:)

!     repeat this for elastin
      Fr_e=inv(Fe0)
      statev(nfib+3:nfib+5)=Fr_e(1,:)
      statev(nfib+6:nfib+8)=Fr_e(2,:)
      statev(nfib+9:nfib+11)=Fr_e(3,:)

      RETURN
      END
