function curvelength = compute_curvelength(coords, varargin)
%coords=[x/y,nodal values]
%the coordinates in the list coords follow a curve, but are not ordered.
%this function seeks the distance between the beginning of the curve until
%the coordinate-couple along the length of the curve. These are returned in
%the list curvelength, that is is in the same order as the coords list.

if max(isnan(coords))
    curvelength=NaN*ones(1,length(squeeze(coords(1,:))));
else
    
    if isempty(varargin)
        center=[mean(coords(1,:)),max(coords(2,:))+0.001];
        %convert to polar coordinate system
        translated_xcoord=coords(1,:)-center(1);
        translated_ycoord=coords(2,:)-center(2);
        phi=atan(translated_xcoord./translated_ycoord);
    else
        coords0=varargin{1};
        center=[mean(coords0(1,:)),max(coords0(2,:))+0.001];
        %convert to polar coordinate system
        translated_xcoord=coords0(1,:)-center(1);
        translated_ycoord=coords0(2,:)-center(2);
        phi=atan(translated_xcoord./translated_ycoord);
    end
[~,order]=sort(phi);
    
ordered_coords=coords(:,order);

n_coords=length(squeeze(coords(1,:)));
%check if ordered coords starts at the hinge or if it needs to be reversed
if ordered_coords(1,1)>-3.3
    ordered_coords=flip(ordered_coords,2);
end

curvelength(1)=0;
for i=2:n_coords
    curvelength(i)=curvelength(i-1)+sqrt((ordered_coords(1,i)-ordered_coords(1,i-1))^2+(ordered_coords(2,i)-ordered_coords(2,i-1))^2);
end

end

