function plot_mid_belly_thickness_over_time(time,coordinates)
%time=[simtype,homeostasis,gain,timepoints]
%coordinates=[simtype,homeostasis,gain,upper/lower,y/z,timepoint,different
%nodes]


%compute the thickness from the two different lines
n_nodes=length(squeeze(coordinates(1,1,1,1,1,1,:)));
n_time =length(squeeze(coordinates(1,1,1,1,1,:,1)));

%compute radial distance and degree of thickening
for i=1:2
    for j=1:2
        for k=1:4
            for t=1:n_time
                radial_distance(i,j,k,t,:)=compute_curvelength(squeeze(coordinates(i,j,k,2,:,t,:)));
                for node=1:n_nodes
                    y_difference=squeeze(coordinates(i,j,k,1,1,t,node)-coordinates(i,j,k,2,1,t,node));
                    z_difference=squeeze(coordinates(i,j,k,1,2,t,node)-coordinates(i,j,k,2,2,t,node));
                    thickness(i,j,k,t,node)=sqrt(y_difference^2+z_difference^2);
                end
                squared_distance_from_center=(max(radial_distance(i,j,k,1,:))/2-squeeze(radial_distance(i,j,k,1,:))).^2;
                [~,middle_indx]=min(squared_distance_from_center);
                thickness_over_time(i,j,k,t)=thickness(i,j,k,t,middle_indx);
            end
       end
    end
end

for i=1:2
    for j=1:2
        for k=1:4
            if i==1
                titlestr='instantanious';
            else
                titlestr='gradual';
            end
            if j==1
                switch k
                    case 1
                        titlestr=strcat(titlestr,' stretch gain 0');
                    case 2
                        titlestr=strcat(titlestr,' stretch gain 2');
                    case 3
                        titlestr=strcat(titlestr,' stretch gain 4');
                    case 4
                        titlestr=strcat(titlestr,' stretch gain 6');
                end
            else
                switch k
                    case 1
                        titlestr=strcat(titlestr,' stress gain 0');
                    case 2
                        titlestr=strcat(titlestr,' stress gain 0.1');
                    case 3
                        titlestr=strcat(titlestr,' stress gain 0.2');
                    case 4
                        titlestr=strcat(titlestr,' stress gain 0.3');
                end
            end
            figure()
            plot(squeeze(time(i,j,k,:)),squeeze(thickness_over_time(i,j,k,:)));
            title(titlestr)
            xlabel('time [days]')
            ylabel('thickness [mm]')
            xlim([1 2500])
            saveas(gcf,strcat(titlestr,'_thickness_over_time.fig'))
            close
        end
    end
end



i=1;
figure
for j=1:2
    for k=1:4
        if j==1
            subplot(1,2,1)
            switch k
                case 1
                    plot(squeeze(time(i,j,k,:)),squeeze(thickness_over_time(i,j,k,:)),"-b");
                    hold on
                case 2
                    plot(squeeze(time(i,j,k,:)),squeeze(thickness_over_time(i,j,k,:)),"--b");
                    hold on
                case 3
                    plot(squeeze(time(i,j,k,:)),squeeze(thickness_over_time(i,j,k,:)),"-.b");
                    hold on
                case 4
                    plot(squeeze(time(i,j,k,:)),squeeze(thickness_over_time(i,j,k,:)),":b");
                    hold on
                    legend('k_\lambda=0/T^i','k_\lambda=2/T^i','k_\lambda=4/T^i','k_\lambda=6/T^i','Location','southeast')
                    xlim([0 2500])
            end
        else
            subplot(1,2,2)
            switch k
                case 1
                    plot(squeeze(time(i,j,k,:)),squeeze(thickness_over_time(i,j,k,:)),"-b");
                    hold on
                case 2
                    plot(squeeze(time(i,j,k,:)),squeeze(thickness_over_time(i,j,k,:)),"--b");
                    hold on
                case 3
                    plot(squeeze(time(i,j,k,:)),squeeze(thickness_over_time(i,j,k,:)),"-.b");
                    hold on
                case 4
                    plot(squeeze(time(i,j,k,:)),squeeze(thickness_over_time(i,j,k,:)),":b");
                    legend('k_\sigma=0/T^i','k_\sigma=0.1/T^i','k_\sigma=0.2/T^i','k_\sigma=0.3/T^i','Location','southeast')
                    xlim([0 2500])
            end
        end        
        saveas(gcf,strcat('all_thickness_over_time.fig'))
    end
end

figure
for i=1:2
    j=2;
    k=4;
    plot(squeeze(time(i,j,k,:)),squeeze(thickness_over_time(i,j,k,:)));
    hold on
    xlim([0 2500])
    xlabel('time')
    ylabel('thickness')
end
legend('instantaneous','gradual')



figure
for j=1:2
    i=2;
    k=4;
    plot(squeeze(time(i,j,k,:)),squeeze(thickness_over_time(i,j,k,:)));
    hold on
    xlim([0 2500])
    xlabel('time')
    ylabel('thickness')
end
legend('stretch','stress')
saveas(gcf,'thickness_over_time_SB3C.fig')
end

