function plot_thickness_profiles(time,coordinates)
%time=[simtype,homeostasis,gain,timepoints]
%coordinates=[simtype,homeostasis,gain,upper/lower,y/z,timepoint,different
%nodes]

%compute the thickness from the two different lines
n_nodes=length(squeeze(coordinates(1,1,1,1,1,1,:)));
n_time =length(squeeze(coordinates(1,1,1,1,1,:,1)));

%the nodes are ordered in a random fashion, but the upper-lower pairing
%seems to match.
%compute initial thickness (equal between the same symtime)

for i=1:2
    y_dif=squeeze(coordinates(i,1,1,1,1,1,:)-coordinates(i,1,1,2,1,1,:));
    z_dif=squeeze(coordinates(i,1,1,1,2,1,:)-coordinates(i,1,1,2,2,1,:));
    thickness_0(i,:)=sqrt(y_dif.^2+z_dif.^2);
end

%compute radial distance and degree of thickening
for i=1:2
    for j=1:2
        for k=1:4
            if i==1
                non_nan_end=27;
            else
                non_nan_end=26;
            end
            radial_distance(i,j,k,:)=compute_curvelength(squeeze(coordinates(i,j,k,2,:,non_nan_end,:)));
            for node=1:n_nodes
                y_difference=squeeze(coordinates(i,j,k,1,1,non_nan_end,node)-coordinates(i,j,k,2,1,non_nan_end,node));
                z_difference=squeeze(coordinates(i,j,k,1,2,non_nan_end,node)-coordinates(i,j,k,2,2,non_nan_end,node));
                thickness(i,j,k,node)=sqrt(y_difference^2+z_difference^2);
            end
            thickening(i,j,k,:)=(squeeze(thickness(i,j,k,:))./squeeze(thickness_0(i,:))'-1)'*100;
            %cosort distances and thicknesses
            [radial_distance(i,j,k,:), order]=sort(squeeze(radial_distance(i,j,k,:)));
            thickening(i,j,k,:)=thickening(i,j,k,order);
        end
    end
end

%compute the evolutions of the radial length over time and plot these
% for a=1:2
%     for j=1:2
%         figure()
%         for k=1:4
%             if a==1
%                 n_time=26;
%             else
%                 n_time=27;
%             end
%             for t=1:n_time
%                 radial_length(a,j,k,t)=max(compute_curvelength(squeeze(coordinates(a,j,k,2,:,t,:))));
%             end
%             plot(squeeze(time(a,j,k,1:26)),squeeze(radial_length(a,j,k,1:26)))
%             hold on
%         end
%         if a==1
%             if j==1
%                 title('instantaneous loading stretch homeostasis')
%                 legend('gain 0', 'gain 2','gain 4','gain 6')
%             else
%                 title('instantaneous loading stress homeostasis')
%                 legend('gain 0.0', 'gain 0.1','gain 0.2','gain 0.3')
%             end
%         else
%             if j==1
%                 title('gradual loading stretch homeostasis')
%                 legend('gain 0', 'gain 2','gain 4','gain 6')
%             else
%                 title('gradual loading stress homeostasis')
%                 legend('gain 0.0', 'gain 0.1','gain 0.2','gain 0.3')
%             end
%         end
%         xlabel('time [days]')
%         ylabel('radial length [mm]')
%     end
% end

%plot thickness vs. radial thickness
% for a=1:2
%     figure()
%     plot(squeeze(radial_distance(a,1,3,:)),squeeze(thickening(a,1,3,:)))
%     hold on
%     plot(squeeze(radial_distance(a,1,4,:)),squeeze(thickening(a,1,4,:)))
%     hold on xlabel('radial distance [mm]'); ylabel('thickening [%]'); if
%     a==1
%         title('instantaneous loading stretch homeostasis')
%     else
%         title('gradual loading stretch homeostasis')
%     end legend('gain 4','gain 6')
%     
%     figure
%     plot(squeeze(radial_distance(a,2,3,:)),squeeze(thickening(a,2,3,:)))
%     hold on
%     plot(squeeze(radial_distance(a,2,4,:)),squeeze(thickening(a,2,4,:)))
%     hold on xlabel('radial distance [mm]'); ylabel('thickening [%]'); if
%     a==1
%         title('instantaneous loading stress homeostasis')
%     else
%         title('gradual loading stress homeostasis')
%     end legend('gain 0.2','gain 0.3')
% end

%make a barplot for the thicknesses
for a=1:2
    figure()
    for j=1:2
        for k=3:4
            squared_distance_from_center=(max(radial_distance(a,j,k,:))/2-squeeze(radial_distance(a,j,k,:))).^2;
            [~,middle_indx]=min(squared_distance_from_center);
            barh((j-1)*2+k-2,thickening(a,j,k,middle_indx))
            hold on
            xlabel('thickness increase [%]')
        end
        
    yticks([1 2 3 4])
    yticklabels({'stretch 4','stretch 6','stress 0.2','stress 0.3'})
    end
    if a==1
        title('instantaneaous load')
        saveas(gcf,'instantaneaous_thick_bars.fig')
        close
    else
        title('gradual load')
        saveas(gcf,'gradual_thick_bars.fig')
        close
    end
end

figure()
for a=1:2
    j=2;
    k=4;
    squared_distance_from_center=(max(radial_distance(a,j,k,:))/2-squeeze(radial_distance(a,j,k,:))).^2;
    [~,middle_indx]=min(squared_distance_from_center);
    barh(a,thickening(a,j,k,middle_indx))
    hold on
    xlabel('thickness increase [%]')
    yticks([1 2])
    yticklabels({'instantaneous','gradual'})
end



%plot profiles
% cmap=colormap(winter);
% for a=1:2
%     for j=1:2
%         for k=1:4
%             figure()
%             for t=1:5:n_time
%                 order=findorder(squeeze(coordinates(a,j,k,1,1,t,:)),squeeze(coordinates(a,j,k,1,2,t,:)));
%                 xcoords=[squeeze(coordinates(a,j,k,1,1,t,order));flip(squeeze(coordinates(a,j,k,2,1,t,order)))];
%                 ycoords=[squeeze(coordinates(a,j,k,1,2,t,order));flip(squeeze(coordinates(a,j,k,2,2,t,order)))];
%                 fill(xcoords,ycoords,cmap(ceil(t/n_time*255),:))
%                 hold on
%                 axis equal
%                 set(gca,'XTick',[]);
%                 set(gca,'YTick',[]);
%                 ylim([-5,9]);
%                 if a==1
%                     titlestr='instantanious';
%                 else
%                     titlestr='gradual';
%                 end
%                 if j==1
%                     switch k
%                         case 1
%                             titlestr=strcat(titlestr,' stretch gain 0');
%                         case 2
%                             titlestr=strcat(titlestr,' stretch gain 2');
%                         case 3
%                             titlestr=strcat(titlestr,' stretch gain 4');
%                         case 4
%                             titlestr=strcat(titlestr,' stretch gain 6');
%                     end
%                 else
%                     switch k
%                         case 1
%                             titlestr=strcat(titlestr,' stress gain 0');
%                         case 2
%                             titlestr=strcat(titlestr,' stress gain 0.1');
%                         case 3
%                             titlestr=strcat(titlestr,' stress gain 0.2');
%                         case 4
%                             titlestr=strcat(titlestr,' stress gain 0.3');
%                     end
%                 end
%                 %title(titlestr)
%             end
%             legend('0','500','1000','1500','2000','2500','Location','northwest','FontSize',12)
%             saveas(gcf,strcat(titlestr,'profiles.fig'))
%             close
%         end
%     end
%     
% end

end